@@ -179,6 +179,10 @@
 # - Include SwapCached to system free and report swap usage
 #   separately in summary
 # - Handle compressed smaps.cap files
+# 2007-11-29:
+# - App memory usage graphs show also SMAPS private memory usage
+# 2008-04-30:
+# - Show optional use-case step description
 # TODO:
 # - Mark reboots more prominently also in report (<h1>):
 #   - dsme/stats/32wd_to -> HW watchdog reboot
@@ -186,8 +190,6 @@
 #   - bootreason -> last boot
 # - Proper option parsing + possibility to state between which
 #   test runs to produce the summaries?
-# - Should the app memory usage changes uses SMAPS instead of RSS?
-#   (it's good to have different sources so that one can compare though)
 # - Show differences in slabinfo and vmstat numbers (pswpin/pswpout)?
 """
 NAME
@@ -198,7 +200,7 @@ SYNOPSIS
 
 DESCRIPTION
 
-This script reads data files produced by the SYTE endurance measurement
+This script reads data files produced by the endurance measurement
 tools.   The data is gathered from proc, X server, SMAPS, syslog etc.
 
 By default all arguments are assumed to be names of directories
@@ -239,7 +241,7 @@ class Colors:
 # color values for memuse, memfree, oom-limit
 bar1colors = ("3149BD", "ADE739", "DE2821")        # blue, green, red
 # color values for rss, size
-bar2colors = ("DE2821", "EAB040")                # red, orange
+bar2colors = ("DE2821", "EAB040", "FBE84A")        # red, orange, yellow
 
 # --------------------- SMAPS data parsing --------------------------
 
@@ -259,7 +261,7 @@ def parse_smaps(filename):
         try:
             line = file.readline()
         except IOError, e:
-            parse_error(write, "ERROR: SMAPS file '%s': %s" % (file, e))
+            syslog.parse_error(write, "ERROR: SMAPS file '%s': %s" % (file, e))
             break
         if not line:
             if sum:
@@ -284,7 +286,7 @@ def parse_smaps(filename):
         if first == '#':
             if line.find("#Pid: ") == 0:
                 pid = line[6:]
-            #print "PID"        #DEBUG
+                #print "PID"        #DEBUG
             continue
         if not pid:
             # sanity check
@@ -305,7 +307,7 @@ def parse_smaps(filename):
         if line.find("Private_Dirty:") == 0:
             amount = int(line[15:-2])
             if code and amount:
-                #print debug_line
+                #print line
                 #print mmap, code, amount
                 private_code += amount
             # Private_Dirty:        0 kB
@@ -829,9 +831,12 @@ def output_memory_graph_table(titles, colors, data):
     "outputs memory usage bars for given (name, (values), tex(t)) tupple array"
     width = 640 # total width of the graph bars
     print "<table><tr>"
+    # column titles
     for title in titles:
         if title:
-            print "<td><i>%s</i></td>" % title
+            print "<th>%s</th>" % title
+        else:
+            print "<th></th>"
     print "</tr>"
     for item in data:
         # row title
@@ -840,12 +845,15 @@ def output_memory_graph_table(titles, colors, data):
         print "<td><table border=0 cellpadding=0 cellspacing=0><tr>"
         for idx in range(len(colors)):
             w = int(item[1][idx]*width)
-            sys.stdout.write('<td bgcolor="%s" width=%d height=16></td>' % (colors[idx], w))
+            if w:
+                sys.stdout.write('<td bgcolor="%s" width=%d height=16></td>' % (colors[idx], w))
         print "</tr></table></td>"
         # texts at end
         for text in item[2]:
             if text:
                 print '<td align="right">%s</td>' % text
+            else:
+                print "<td></td>"
         print "</tr>"
     print "</table>"
 
@@ -853,8 +861,10 @@ def output_memory_graph_table(titles, colors, data):
 def output_apps_memory_graphs(cases):
     "outputs memory graphs bars for the individual processes"
     # arrange per use-case data to be per pid
+    smaps_available = 0
     rounds = 0
     data = {}
+    # get names and pids
     for testcase in cases:
         commands = testcase['commands']
         processes = testcase['processes']
@@ -871,6 +881,14 @@ def output_apps_memory_graphs(cases):
                 data[namepid] = {}
                 data[namepid]['first'] = rounds
             # process is also in this round, so add its info
+            sum = 0
+            if 'smaps' in testcase:
+                if pid in testcase['smaps']:
+                    sum = testcase['smaps'][pid]
+                    smaps_available = 1
+                else:
+                    syslog.parse_error(sys.stdout.write, "WARNING: SMAPS data missing for pid %s" % pid)
+            process['SMAPS'] = sum
             data[namepid][rounds] = process
         rounds += 1
 
@@ -880,11 +898,18 @@ def output_apps_memory_graphs(cases):
     largest_size = 0
     for namepid in data:
         changerounds = pidrounds = 0
-        max_size = max_rss = 0
-        min_size = min_rss = 128*1024
+        max_size = max_dirty = 0
+        min_size = min_dirty = 512*1024
         for idx in range(rounds):
             if idx in data[namepid]:
-                rss = data[namepid][idx]['VmRSS']
+                if smaps_available:
+                    dirty = data[namepid][idx]['SMAPS']
+                else:
+                    dirty = data[namepid][idx]['VmRSS']
+                if dirty < min_dirty:
+                    min_dirty = dirty
+                if dirty > max_dirty:
+                    max_dirty = dirty
                 size = data[namepid][idx]['VmSize']
                 if size < min_size:
                     if pidrounds:
@@ -894,25 +919,26 @@ def output_apps_memory_graphs(cases):
                     if pidrounds:
                         changerounds += 1
                     max_size = size
-                if rss < min_rss:
-                    min_rss = rss
-                if rss > max_rss:
-                    max_rss = rss
-                if rss > max_rss:
-                    max_rss = rss
                 pidrounds += 1
         if pidrounds > 1:
-            rss_change = (float)(max_rss - min_rss) / max_rss / pidrounds
+            # if SMAPS data available, dirty is private dirty memory,
+            # otherwise it's RSS. Size = VmSize
+            if max_dirty:
+                dirty_change = (float)(max_dirty - min_dirty) / max_dirty / pidrounds
+            else:
+                if smaps_available:
+                    syslog.parse_error(sys.stdout.write, "WARNING: no SMAPS dirty for %s[%s]. Disable swap and try again\n\t(SMAPS doesn't work properly with swap)" % namepid)
+                dirty_change = 0
             size_change = (float)(max_size - min_size) / max_size / pidrounds
-            # if >0.2% memory change per round in RSS or Size, or
+            # if >0.2% memory change per round in dirty or Size, or
             # size changes on more than half of the rounds, add to list
-            if rss_change > 0.002 or size_change > 0.002 or 2*changerounds > pidrounds:
-                sizes.append((max_rss,namepid))
+            if dirty_change > 0.002 or size_change > 0.002 or 2*changerounds > pidrounds:
+                sizes.append((max_dirty,namepid))
         if max_size > largest_size:
             largest_size = max_size
     largest_size = float(largest_size)
     
-    # first sort according to the RSS size
+    # first sort according to the dirty (or RSS) size
     sizes.sort()
     sizes.reverse()
     # then sort according to names
@@ -920,20 +946,29 @@ def output_apps_memory_graphs(cases):
     for size in sizes:
         namepid = size[1]
         # sorting order is: name, first round for pid, pid
-        orders.append((namepid[0], data[namepid]['first'], namepid[1], namepid))
+        orders.append((namepid[0], data[namepid]['first'], namepid[1]))
     del(sizes)
     orders.sort()
     
     # amount of memory in the device (float for calculations)
     print """
-<p>Only processes which resident size changes during tests are listed.
-If process has same name and (max) RSS as some other process, it's
-assumed to be a thread and ignored.
+<p>Only processes which VmSize and amount of private dirty memory
+changes during tests are listed.  If a process has same name and size
+as its parent, it's assumed to be a thread and ignored.
+
+<p>Note: Memory is not any more accounted as dirty if it's swapped out
+(even when it's paged back in), this seems like a kernel (2.6.21) bug.
+RSS can decrease if device is just running low on memory because
+kernel can just discard unmodified/unused pages. Size tells amount of
+all virtual allocations and memory maps of a process, so it might not
+have any relation to real process memory usage. However, it can show
+leaks which cause process eventually to run out of (2GB) address space
+(e.g. if it's not collecting thread resources).
 """
     for order in orders:
-        namepid = order[3]
+        namepid = (order[0],order[2])
         process = data[namepid]
-        print "<h4>%s [%s]</h4>" % namepid
+        print "<h4><i>%s [%s]</i></h4>" % namepid
         text = ''
         busy = 0
         prev_idx = 0
@@ -942,16 +977,24 @@ assumed to be a thread and ignored.
         for idx in range(rounds):
             if idx in process:
                 item = process[idx]
+
                 rss = item['VmRSS']
                 size = item['VmSize']
-                sizes = (rss/largest_size, (size - rss)/largest_size)
+                if smaps_available:
+                    dirty = item['SMAPS']
+                    if rss < dirty:
+                        syslog.parse_error(sys.stdout.write, "WARNING: %s[%s] RSS (%s) < SMAPS dirty (%s)" % (namepid + (rss, dirty)))
+                        rss = dirty
+                    text = ["%skB" % dirty, "%skB" % rss, "%skB" % size]
+                else:
+                    text = ["", "%skB" % rss, "%skB" % size]
+                    dirty = 0
+                sizes = (dirty/largest_size, (rss-dirty)/largest_size, (size-rss)/largest_size)
 
                 sleepavg = int(item['SleepAVG'][:-1])
                 if sleepavg < 90:
                     busy = 1
-                    text = ("%skB" % rss, "%skB" % size, "(%d%%)" % sleepavg)
-                else:
-                    text = ("%skB" % rss, "%skB" % size)
+                    text.append("(%d%%)" % sleepavg)
 
                 if idx:
                     if text == prev_text:
@@ -969,8 +1012,7 @@ assumed to be a thread and ignored.
                 if text == nan:
                     # previous one didn't have anything either
                     continue
-                prev_rss = prev_size = 0
-                sizes = (0,0)
+                sizes = (0,0,0)
                 text = nan
                 case = "---"
             columndata.append((case, sizes, text))
@@ -978,7 +1020,9 @@ assumed to be a thread and ignored.
             sleeptext = "Sleep:"
         else:
             sleeptext = None
-        titles = ("Test-case:", "Graph", "RSS:", "Size:", sleeptext)
+        titles = ["Test-case:", "Graph", "Dirty:", "RSS:", "Size:", sleeptext]
+        if not smaps_available:
+            titles[2] = ""
         output_memory_graph_table(titles, bar2colors, columndata)
 
 
@@ -1088,8 +1132,13 @@ def output_html_report(data):
 <li>Resource usage changes for each of the test rounds:
   <ul>
 """ % (title, title)   #" fool Jed syntax highlighter
-    for round in range(len(data)-1):
-        print '  <li><a href="#round-%d">Round %d</a>' % (round+1, round+1)
+    for round in range(rounds):
+        idx = round + 1
+        if data[idx].has_key('description') and data[idx]['description']:
+            desc = " (%s)" % data[idx]['description']
+        else:
+            desc = ""
+        print '  <li><a href="#round-%d">Round %d</a>%s' % (idx, idx, desc)
     print """
   </ul>
 <li>Summary of changes between all the rounds after the initial one:
@@ -1172,6 +1221,7 @@ def parse_syte_stats(dirs):
     """parses given CSV files into a data structure"""
     data = []
     for dirname in dirs:
+        # get basic information
         file = "%s/usage.csv" % dirname
         sys.stderr.write("Parsing '%s'...\n" % file)
         items = parse_csv(file)
@@ -1185,6 +1235,7 @@ def parse_syte_stats(dirs):
             if not (os.path.exists(file)):
                 file = None
         if file:
+            # get system SMAPS memory usage data
             sys.stderr.write("Parsing '%s'...\n" % file)
             items['smaps'], items['private_code'] = parse_smaps(file)
             if not items['smaps']:
@@ -1200,8 +1251,13 @@ def parse_syte_stats(dirs):
             # get the crashes and other errors
             sys.stderr.write("Parsing '%s'...\n" % file)
             items['errors'] = syslog.parse_syslog(sys.stdout.write, file)
-            items['logfile'] = file
-        #print items
+            items['logfile'] = file 
+
+        file = "%s/step.txt" % dirname
+        if os.path.exists(file):
+            # use-case step description
+            items['description'] = open(file).read().strip()
+
         data.append(items)
     return data
 
